require("dotenv").config()
var Producer = require("../libs/Producer")
var MessageDef = require("crawler-message-definition")
var underscore = require("underscore")
var CurrentMetadata = require("../libs/MetaRiser").getMetadata()
var Emitter = new (require("events").EventEmitter)
var MASTER_PORT = process.env.MASTER_PORT
var MASTER_IP_ADDRESS = process.env.MASTER_IP_ADDRESS
var GlobalTimeoutId = -1
var HEARTBEAT_DELAY = 1000 * 2
var SLAVE_ID = null
var Module = {}
var errorOnHeartbeat = false

//@Private
function startHeartBeat(){
	var message = new MessageDef.HeartBeat(
		SLAVE_ID,
		CurrentMetadata.ip_address,
		CurrentMetadata.port,
		CurrentMetadata.social_media,
		new Date().getTime()
	)
	Producer.ask({
		ip_address : MASTER_IP_ADDRESS,
		port : MASTER_PORT
	},message)
	.then(function(reply){
		if ( MessageDef.isTypeOf("Reply",reply) ){
			var data = reply.data
			if ( data ) {
				if ( errorOnHeartbeat ) {
					console.log("Master has re-acknowledge heartbeat!")
					errorOnHeartbeat = false
				}
				GlobalTimeoutId = setTimeout(startHeartBeat,HEARTBEAT_DELAY)
			} else {
				Emitter.emit("heartbeat_rejected")
			}
		}
	})
	.catch(function(reason){
		console.log("Error on sending heartbeat "+reason)
		errorOnHeartbeat = true
		GlobalTimeoutId = setTimeout(startHeartBeat,HEARTBEAT_DELAY)
	})
}


function start(slaveId){
	SLAVE_ID = slaveId
	if ( GlobalTimeoutId == -1) {
		startHeartBeat()	
	}	
}

function stop(){
	clearTimeout(GlobalTimeoutId)
	SLAVE_ID = null
	GlobalTimeoutId = -1
}

function onHeartBeatRejected(fn) {
	Emitter.on("heartbeat_rejected",fn)
}

Module.onHeartBeatRejected = onHeartBeatRejected
Module.start = start
Module.stop = stop
module.exports = Module