require("dotenv").config()
var MessageDef = require("crawler-message-definition")
var Consumer = require("./libs/Consumer")
var Producer = require("./libs/Producer")
var MetaRiser = require("./libs/MetaRiser")
var PaceMaker = require("./aspects/PaceMaker")
var underscore = require("underscore")
var EventEmitter = require("events").EventEmitter

var MASTER_PORT = process.env.MASTER_PORT
var MASTER_IP_ADDRESS = process.env.MASTER_IP_ADDRESS
var SLAVE_ID = null
var MASTER_SEARCH_DELAY = process.env.MASTER_SEARCH_DELAY || 30000
var CurrentMetadata = MetaRiser.getMetadata(true)
var isMasterFound = false
var Emitter = new EventEmitter
var Module= {}

var EVENT_AGENT_READY = "agent_ready"
var EVENT_REQUEST = "request"
var EVENT_CONSUMER_READY = "consumer_ready"
var EVENT_AGENT_SUICIDE = "agent_suicide"


var MasterAddress = {
	ip_address : MASTER_IP_ADDRESS,
	port : MASTER_PORT
}

function findMaster(){
	var message = new MessageDef.RegisterSlave(
		CurrentMetadata.ip_address,
		CurrentMetadata.port,
		CurrentMetadata.social_media,
		CurrentMetadata.crawler_type,
		CurrentMetadata.slave_type
	)
	Producer
	.sendMessage(MasterAddress,message)
	.then(function(reply){

		if ( MessageDef.isTypeOf("Reply",reply)){
			var data = reply.data
			if ( data ) {				
				SLAVE_ID = data
				isMasterFound = true
				console.log("Connected to Master! Id :",SLAVE_ID)
				Emitter.emit(EVENT_AGENT_READY,SLAVE_ID)
				PaceMaker.start(SLAVE_ID)
			}
			if(!isMasterFound){
				console.log("Master not accepting registration. Retrying..")
				setTimeout(function(){
					findMaster()
				},MASTER_SEARCH_DELAY)
			}
		} else if(MessageDef.isTypeOf("ErrorReply",reply)){
			console.log("Slave encountered fatal error on registering to Master",reply.message)
			throw new Error(reply.message)
		}

	})
	.catch(function(err){
		console.log("Failure to contact Master "+err)
		console.log("Retrying..")
		setTimeout(function(){
			findMaster()
		},MASTER_SEARCH_DELAY)
	})

}

function onMessage(eventName,fn){
	Consumer.on(EVENT_REQUEST,function(message){
		if ( message.responder ){
			var realMessage = message.message
			var responder = message.responder
			if(MessageDef.isTypeOf(eventName,realMessage) && underscore.isFunction(fn)){
				fn(realMessage,responder)
			}
		} else {
			if(MessageDef.isTypeOf(eventName,message) && underscore.isFunction(fn)){
				fn(message)
			}
		}		
	})	
	return this
}

function emitMessage(type,body){
	var wrappedMessage = MessageDef.createMessage(type,body)
	Producer.sendMessage(MasterAddress,wrappedMessage)
	return this;
}

function main(){
	Consumer.init(CurrentMetadata)
	Producer.init(CurrentMetadata)
	Consumer.on("listening",function(){
		if(CurrentMetadata.role == "slave"){
			console.log("Begin searching for master on",MasterAddress)
			findMaster()
		} else {
			Emitter.emit(EVENT_AGENT_READY)
		}	
	})
	if ( CurrentMetadata.role == "slave" ) {
		PaceMaker.onHeartBeatRejected(function(){
			console.log("Heartbeat is rejected by Master!")
			console.log("Perform suicide..")
			Emitter.emit(EVENT_AGENT_SUICIDE)
			SLAVE_ID = null
			PaceMaker.stop()
			findMaster()
		})
	}
}

Module.on = onMessage
Module.emit = emitMessage
Module.onAgentSuicide = function( fn ) {
	Emitter.on(EVENT_AGENT_SUICIDE, fn )
	return this;
}
Module.getMasterIdentity = function ( ) {
	return MasterAddress
}
Module.ask = function( target, message ) {
	return Producer.ask( target, message )
}
Module.tell = function( target, message) {
	return Producer.tell( target, message )
}
Module.MessageDef = MessageDef 
main()
module.exports = Module









