var http = require("http")
var EventEmitter = require("events").EventEmitter
var underscore  = require("underscore")
var MessageDef = require("crawler-message-definition")
var CurrentMetadata = {}
var ServerInstance = null

var EVENT_REQUEST = "request"
var EVENT_LISTENING = "listening"

var okMessage = new MessageDef.Reply(true)
var errorMessage = new MessageDef.ErrorReply()
var Module = new EventEmitter

function responseFormatter(response,content,status){
	response.writeHead(status,{
		"Content-Type":"application/json"
	})
	response.write(content.toString())
	response.end()
}

function createMessageConsumer(port,ipAddress,requestHandler){
	var server = http.createServer(requestHandler)
	server.listen(port,ipAddress,function(){
		console.log("Crawler-http-agent listening on",ipAddress+":"+port)
		Module.emit(EVENT_LISTENING)
	})
	return server;
}

function main(metaData){

	var that = this
	var ipAddress = metaData.ip_address;
	var port = metaData.port
	var bindAllInterfaces = metaData.bind_all_interfaces
	if( bindAllInterfaces ) {
		ipAddress = "0.0.0.0"
	}
	CurrentMetadata = metaData	
	ServerInstance = createMessageConsumer(port,ipAddress,function(request,response){
		var bufferChunk = []
		request
		.on("data",function(buffer){
			bufferChunk.push(buffer)
		})
		.on("end",function(){
			var rawRequest = bufferChunk.join("")
			var parsedRawMessage = JSON.parse(rawRequest)
			var header = parsedRawMessage.header
			var message = MessageDef.unserialize(parsedRawMessage.body)
			var needReply = message.needReply || (header.pattern == "ask" )
			if ( !needReply ){
				responseFormatter(
					response,
					okMessage,
					200
				)	
				that.emit(EVENT_REQUEST,message)
			} else {
				that.emit(EVENT_REQUEST,{
					message : message,
					responder : function(data){
						responseFormatter(response,data,200)
					}
				})
			}
		})
		.on("error",function(){
			responseFormatter(
				response,
				new MessageDef.ErrorReply("Malformed request"),
				400
			)
		})

	})

}

Module.init = main
module.exports = Module